untyped
global function mapLightingInit
global function mapLightingSettings

array <array <string> > mapNameList = [
    ["mp_lobby", "Lobby"],
    ["mp_angel_city", "Angel City"],
    ["mp_black_water_canal", "Blackwater"],
    ["mp_coliseum", "Coliseum"],
    ["mp_coliseum_column", "Pillars"],
    ["mp_colony02", "Colony"],
    ["mp_complex3", "Complex"],
    ["mp_crashsite3", "Crashsite"],
    ["mp_drydock", "Drydock"],
    ["mp_eden", "Eden"],
    ["mp_forwardbase_kodai", "Kodai"],
    ["mp_glitch", "Glitch"],
    ["mp_grave", "Boomtown"],
    ["mp_homestead", "Homestead"],
    ["mp_lf_deck", "Deck"],
    ["mp_lf_meadow", "Meadow"],
    ["mp_lf_stacks", "Stacks"],
    ["mp_lf_township", "Township"],
    ["mp_lf_traffic", "Traffic"],
    ["mp_lf_uma", "Uma"],
    ["mp_relic02", "Relic"],
    ["mp_rise", "Rise"],
    ["mp_thaw", "Exoplanet"],
    ["mp_wargames", "Wargames"]
]

void function mapLightingSettings(){
    #if UI
        #if S2_MAP_LIGHTING_MOD_SETTINGS
            AddModTitle("S2.MapLighting")
            AddModCategory("Main")
            AddConVarSettingEnum("s2_ml_rgb_all", "RGB Cycle on Enabled Maps", ["Off", "On :D"])
            AddConVarSettingEnum("s2_ml_uni_col", "Uniform Colour on Enabled Maps", ["Off", "On"])
            AddConVarSetting("s2_ml_sun_col", "Uniform Sun Colour", "vector")
            AddConVarSetting("s2_ml_sky_col", "Uniform Sky Colour", "vector")
            foreach(pair in mapNameList){
                if( !(pair[0] == "mp_angel_city" && (S2_MAP_LIGHTING_ANGEL_CITY_OH || S2_MAP_LIGHTING_RWYN_NC_LIGHT)) && !(pair[0] == "mp_glitch" && S2_MAP_LIGHTING_GLITCH_OH) && !(pair[0] == "mp_wargames" &&  S2_MAP_LIGHTING_WARGAMES_OH)){
                    AddModCategory(pair[1])
                    AddConVarSettingEnum(("s2ml_" + pair[0] + "_enabled"), (pair[1] + " Enabled"), ["Off", "On"])
                    AddConVarSettingEnum(("s2ml_" + pair[0] + "_enabled_reset"), (pair[1] + " Reset Enabled"), ["Off", "On"])
                    AddConVarSetting(("s2ml_" + pair[0] + "_sun"), (pair[1] + " Sun Colour"), "vector")
                    AddConVarSetting(("s2ml_" + pair[0] + "_sky"), (pair[1] + " Sky Colour"), "vector")
                }
                else{
                    SetConVarInt("s2ml_" + pair[0] + "_enabled", 0)
                    SetConVarInt("s2ml_" + pair[0] + "_enabled_reset", 0)
                }
            }
        #else
            thread noSettingsDiagThread()
        #endif
    #endif
}

#if UI

void function noSettingsDiagThread(){
    WaitFrame()
    noSettingsDiag()
}

void function noSettingsDiag(){
    DialogData errorPopUp
    errorPopUp.header = "MISSING REQUIRED DEPENDENCY"
    errorPopUp.message = "MapLighting requires ModSettings to function.\nThe mod was not found; please download it."
    errorPopUp.image = $"ui/menu/common/dialog_error"
    AddDialogButton(errorPopUp, "Exit", void function(){ ClientCommand("exit") })
    AddDialogButton(errorPopUp, "Download", void function(){
        LaunchExternalWebBrowser("https://northstar.thunderstore.io/package/EladNLG/ModSettings/", WEBBROWSER_FLAG_FORCEEXTERNAL)
        noSettingsDiag()
    })
    OpenDialog(errorPopUp)
}

#endif

void function mapLightingInit(){
    #if CLIENT
        thread mapLightingMain()
    #endif
}

#if CLIENT

vector function GetConVarFloat3(string convar){
    array<string> value = split(GetConVarString(convar), " ")
    try{
        return Vector(value[0].tofloat(), value[1].tofloat(), value[2].tofloat()) 
    }
    catch(ex){
        throw "Invalid convar " + convar + "! make sure it is a float3 and formatted as \"X Y Z\""
    }
    unreachable
}

string function vectorToConVar(vector input){
    return (input.x.tostring() + " " + input.y.tostring() + " " + input.z.tostring())
}

bool function isCustomMap(string mapName){
    foreach(pair in mapNameList){
        if(pair[0] == mapName)
            return false
    }
    return true
}

void function mapLightingMain(){
    while(2){
        WaitFrame()
        entity player = GetLocalClientPlayer()
        if(!IsValid(player) || player == null || IsWatchingReplay())
            continue
        string currentMapName = GetMapName()
        if(isCustomMap(currentMapName))
            continue
        int currentEnabled = GetConVarInt("s2ml_" + currentMapName + "_enabled")
        int currentResetEnabled = GetConVarInt("s2ml_" + currentMapName + "_enabled_reset")
        if(currentEnabled){
            if(GetConVarInt("s2_ml_rgb_all")){
                vector rainbow
                rainbow.x = sin(Time() * PI) + 1
                rainbow.y = sin((0.5*Time() + 1.0/3.0) * PI * 2) + 1
                rainbow.z = sin((0.5*Time() + 2.0/3.0) * PI * 2) + 1
                SetConVarString("mat_sun_color", vectorToConVar(rainbow))
                SetConVarString("mat_sky_color", vectorToConVar(rainbow))
            }
            else{
                string currentSunColour = vectorToConVar(GetConVarInt("s2_ml_uni_col") ? GetConVarFloat3("s2_ml_sun_col") : GetConVarFloat3("s2ml_" + currentMapName + "_sun"))
                string currentSkyColour = vectorToConVar(GetConVarInt("s2_ml_uni_col") ? GetConVarFloat3("s2_ml_sky_col") : GetConVarFloat3("s2ml_" + currentMapName + "_sky"))
                if(GetConVarString("mat_sun_color") != currentSunColour)
                    SetConVarString("mat_sun_color", currentSunColour)
                if(GetConVarString("mat_sky_color") != currentSkyColour)
                    SetConVarString("mat_sky_color", currentSkyColour)
            }
        }
        else{
            if(currentResetEnabled){
                SetConVarString("mat_sun_color", "0 0 0 0")
                SetConVarString("mat_sky_color", "0 0 0 0")
            }
        }
    }
}

#endif